"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.encrypt = void 0;
const crypto_1 = __importDefault(require("crypto"));
const path_1 = require("path");
const asar_1 = require("asar");
const helpers_1 = require("./helpers");
const promises_1 = require("fs/promises");
// TODO: encrypt files from existing asar archive.
// See: https://github.com/sleeyax/asarmor/issues/42
/**
 * Encrypts and packages all files into an asar archive.
 */
async function encrypt({ keyFilePath = path_1.join(__dirname, 'key.txt'), key: keyPlaintext, src, dst, }) {
    const key = keyPlaintext
        ? Buffer.from(keyPlaintext)
        : Buffer.from(helpers_1.fromHex(await promises_1.readFile(keyFilePath)));
    return asar_1.createPackageWithOptions(src, dst, {
        unpack: '*.node',
        transform(filename) {
            if (path_1.extname(filename) == '.js') {
                // generate random 16-byte initialization vector (IV)
                const iv = crypto_1.default.randomBytes(16);
                // whether we have already prefixed the first chunk of content with the IV
                let hasPrefix = false;
                const cipher = crypto_1.default.createCipheriv('aes-256-cbc', key, iv);
                cipher.setAutoPadding(true);
                cipher.setEncoding('base64');
                // monkey patch push method to put the IV in front of the encrypted data
                const _p = cipher.push;
                cipher.push = function (chunk, enc) {
                    if (!hasPrefix && chunk != null) {
                        hasPrefix = true;
                        return _p.call(this, Buffer.concat([iv, chunk]), enc);
                    }
                    else {
                        return _p.call(this, chunk, enc);
                    }
                };
                return cipher;
            }
        },
    });
}
exports.encrypt = encrypt;
